/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Forecastfox.
 *
 * The Initial Developer of the Original Code is
 * Jon Stritar <jstritar@MIT.EDU>.
 * Portions created by the Initial Developer are Copyright (C) 2005
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 * Jon Stritar <jstritar@MIT.EDU>
 * Richard Klein <richwklein@mchsi.com>
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

/* XPCOM Constants */
const CLASS_ID = Components.ID("{27e1bca0-aded-11d9-9669-0800200c9a66}");
const CLASS_NAME = "Forecastfox Profiles Component";
const CONTRACT_ID = "@ensolis.com/forecastfox/profiles;1";
const nsIDOMXPathResult = Components.interfaces.nsIDOMXPathResult;
const nsIFilePicker = Components.interfaces.nsIFilePicker;
const ffIError = Components.interfaces.ffIError;
const ffIProfiles = Components.interfaces.ffIProfiles;
const ffIDisk = Components.interfaces.ffIDisk;

/******************************************************************************
 * ffProfiles Component
 ******************************************************************************/
function ffProfiles()
{
  this.wrappedJSObject = this;
};

ffProfiles.prototype = {
  _batch: false,
  _loading: false,
  _current: null,
  _rotating: false,
  _xpath: null,
  _xresolve: null,
  _obs: null,
  _doc: null,
  _branch: null,
  _default: null,
  _excludes: null,
  _timer: null,
  _bundle: null,
  _restart: null,
  _manager: null,
  
  get batch() { return this._batch; },
  get current() { 
    if (!this._current)
      return "";
      
    return this._current;  
  },
  
  start: function()
  {
    try {
      this._loading = true;
      
      //set up components
      var pbs = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefService);
      var sbs = Components.classes["@mozilla.org/intl/stringbundle;1"].getService(Components.interfaces.nsIStringBundleService);
      this._xpath = Components.classes["@mozilla.org/dom/xpath-evaluator;1"].getService(Components.interfaces.nsIDOMXPathEvaluator);
      this._xresolve = Components.classes["@ensolis.com/forecastfox/resolver;1"].createInstance(Components.interfaces.nsIDOMXPathNSResolver);
      this._obs = Components.classes["@mozilla.org/observer-service;1"].getService(Components.interfaces.nsIObserverService);
      this._manager = Components.classes["@ensolis.com/forecastfox/manager;1"].getService(Components.interfaces.ffIManager);
      this._timer = Components.classes["@mozilla.org/timer;1"].createInstance(Components.interfaces.nsITimer);
      this._bundle = sbs.createBundle("chrome://forecastfox/locale/forecastfox.properties");    
      this._branch = pbs.getBranch("forecastfox.");
      this._default = pbs.getDefaultBranch("forecastfox.");
      
      //setup excludes
      this._excludes = new Object();
      this._excludes["profile.current"] = true;
      this._excludes["profile.switch.enabled"] = true;
      this._excludes["profile.switch.delay"] = true;
      this._excludes["migrated"] = true;
      this._excludes["pinged"] = true;
      this._excludes["schedule.span"] = true;
      this._excludes["schedule.bump"] = true;
 
       //load and migrate data
      this._restart = this._createError(ffIError.SEVERITY_INFO, null, "", "");   
      this._loadDoc();
      
      //start rotating
      if (this._restart.severity != ffIError.SEVERITY_ERROR)
        this.rotate(ffIProfiles.ROTATE_START); 
     
      this._loading = false;
      return this._restart;
    } catch (e) {
      this._manager.disk.recordErrorMsg("An error occured while trying to load the profiles module. Exception details: "+e);
    }
  },
  
  stop: function()
  {  
    //save the current profile
    this.save(this._current);

    //unlock pref
    this._branch.unlockPref("migrated");
    
    //destroy variables
    this._batch = null;
    this._loading = null;
    this._current = null;
    this._rotating = null;        
    this._xpath = null;
    this._xresolve = null;
    this._doc = null;
    this._obs = null;
    this._branch = null;
    this._default = null;
    this._excludes = null; 
    this._bundle = null;
    this._restart = null;
    this._manager = null;
  },
  
  load: function(aName)
  {
    this._batch = true;
    this.rotate(ffIProfiles.ROTATE_STOP);
    this._manager.cancel();  
      
    //save the current profile
    if (this._isProfile(this._current))
      this.save(this._current);

    //get profile
    var profiles = this._evalPath(this._doc, "//profiles");
    var profile = this._evalPath(profiles, "./profile[@id='" + aName + "']");

    //load preferences
    var prefs = this._getPrefs();
    for (var x=0; x<prefs.length; x++) {
      if (this._excludes[prefs[x]])
        continue;
      var pref = this._evalPath(profile, "./pref[@name='" + prefs[x] + "']");
      this._setPref(pref);
    };
    
    //set current
    this._current = aName;   
    
    //notify complete
    this._batch = false;
    if (!this._loading) {
      this._obs.notifyObservers(this, "forecastfox:profile", aName);
      this.rotate(ffIProfiles.ROTATE_START);          
    }
  },
  
  save: function(aName)
  {
    //return if nothing passed
    if (!aName)
      return;
      
    var profiles = this._evalPath(this._doc, "//profiles");
    var profile = this._evalPath(profiles, "./profile[@id='" + aName + "']");
    
    //check if this is a new profile
    if (profile)
      profile.parentNode.removeChild(profile);

    //create profile
    profile = this._doc.createElement("profile");
    profile.setAttribute("id", aName);
    
    //add preference to it
    var prefs = this._getPrefs();
    var pref, type;
    for (var x=0; x<prefs.length; x++) {
      if (this._excludes[prefs[x]])
        continue;    
        
      //if we migrated a pref may exists with no value
      //so we wrap this in a try catch
      try {
        type = this._getPrefType(prefs[x]);
        pref = this._doc.createElement("pref");
        pref.setAttribute("name", prefs[x]);
        pref.setAttribute("type", type);
        pref.setAttribute("value", eval("this._branch.get" + type + "Pref('" + prefs[x] + "')"));
        profile.appendChild(pref);
      } catch(e) {};
    };
    
    //append profile to profiles
    profiles.appendChild(profile);

    //flush to disk
    var file = this._manager.disk.get("profiles.xml", ffIDisk.TYPE_CACHE);
    this._manager.disk.write(this._doc, file, true);
  },

  rotate: function(aRotate)
  {
    //cancel in timers
    this._timer.cancel();

    //stop rotating    
    if (aRotate == ffIProfiles.ROTATE_STOP)
      this._rotating = false;
    else {
      this._rotating = true;
      this._schedule();
    }
  },

  create: function(aName)
  {
    this._batch = true;
    this.rotate(ffIProfiles.ROTATE_STOP);
    this._manager.cancel();   
    
    //make sure profile is saved
    if (this._isProfile(this._current))
      this.save(this._current);

    //clone the current profile
    var name = this._stripName(aName);
    var profiles = this._evalPath(this._doc, "//profiles");
    var profile = this._evalPath(profiles, "./profile[@id='" + this._current + "']");
    if (!profile)
      profile = this._doc.createElement("profile");
    var to = profile.cloneNode(true);
            
    //set id
    to.setAttribute("id", name);
        
    //copy cache
    this._cacheFiles(profile, to, false);
    this._cachePrefs(profile, to);
    
    //append
    profiles.appendChild(to);
        
    //set current pref which will
    //load this one and flush data to disk
    this._batch = false;
    this.rotate(ffIProfiles.ROTATE_START);    
    this._branch.setCharPref("profile.current", name);       
  },
  
  rename: function(aName, aTo)
  {
    this._batch = true;
    this.rotate(ffIProfiles.ROTATE_STOP);
    this._manager.cancel();   
    
    //make sure profile is saved
    if (this._isProfile(this._current))
      this.save(this._current);

    //clone the from profile
    var name = this._stripName(aTo);
    var profiles = this._evalPath(this._doc, "//profiles");
    var profile = this._evalPath(profiles, "./profile[@id='" + aName + "']");
    var to = profile.cloneNode(true);
   
    //set id
    to.setAttribute("id", name);
        
    //copy cache
    this._cacheFiles(profile, to, true);
    this._cachePrefs(profile, to);
    
    //append
    profiles.appendChild(to); 
    
    //remove from
    profiles.removeChild(profile);
    
    //set current pref which will
    //load this one and flush data to disk
    this._current = null;  
    this._batch = false;
    this.rotate(ffIProfiles.ROTATE_START);        
    this._branch.setCharPref("profile.current", name);  
  },
  
  remove: function(aName, aTo)
  {
    this._batch = true;
    this.rotate(ffIProfiles.ROTATE_STOP);
    this._manager.cancel();   
          
    //make sure profile is saved
    if (this._isProfile(this._current))
      this.save(this._current);
      
    //get the from profile
    var profiles = this._evalPath(this._doc, "//profiles");
    var profile = this._evalPath(profiles, "./profile[@id='" + aName + "']");
    
    //delete cache files
    this._cacheFiles(profile, null, true);
    
    //remove from
    profiles.removeChild(profile);
    
    //set current pref which will
    //load this one and flush data to disk
    this._current = null;
    this._batch = false;
    this.rotate(ffIProfiles.ROTATE_START);        
    this._branch.setCharPref("profile.current", aTo);    
  },
  
  getProfiles: function(aCount)
  {
    var profiles = this._evalPath(this._doc, "//profiles");
    var val = new Array();

    profiles = profiles.getElementsByTagName("profile");  
    for (var i=0; i<profiles.length; i++)
      val.push(profiles[i].getAttribute("id"));
    
    val.sort();  
    aCount.value = val.length;
     
    return val;    
  },
  
  open: function(aOpen)
  {
    //set variables
    var alerts = Components.classes["@mozilla.org/embedcomp/prompt-service;1"].getService(Components.interfaces.nsIPromptService);
    var windowManager = Components.classes["@mozilla.org/appshell/window-mediator;1"].getService().QueryInterface(Components.interfaces.nsIWindowMediator);
    var topWindow = windowManager.getMostRecentWindow(null);
    var doc, profiles, node, current, prefs, pref, type, parent, version, x;
        
    //get file
    var file = this._filePicker(aOpen, topWindow);
    if (!file)
      return;
    
    //do import-export  
    switch (aOpen) {
      case ffIProfiles.OPEN_EXPORT:
        this._export(alerts, topWindow, file);
        break;
      case ffIProfiles.OPEN_IMPORT:
        this._import(alerts, topWindow, file);
        break;
    }
  },
     
  _loadDoc: function()
  {
    this._batch = true;
    var from = this._branch.getCharPref("migrated");
    var previous = this._branch.prefHasUserValue("migrated");
    var file = this._manager.disk.get("profiles.xml", ffIDisk.TYPE_CACHE);
    var exists = file.exists();
    var doc, pFrom;
    
    /*::::: do simple migration :::::*/
    if (previous) {
        
      // completes the uninstallation/upgrade of weatherfox
      if (from == "*") {
        this._removeBranch("weatherfox.", false);
        this._manager.disk.remove(ffIDisk.TYPE_WEATHERFOX);      
      };
      
      //get profile document
      doc = this._getDoc(file);
      if (!doc)
        return;
              
      //do any transform if needed
      pFrom = this._evalPath(doc, "//@version");
      if (pFrom) 
        pFrom = pFrom.textContent;
      else
        pFrom = from;
      doc = this._transform(doc, pFrom);
      if (!doc) {
        this._restart = this._createError(ffIError.SEVERITY_ERROR, "ff.transform", null, null);
        return;
      };
      
      //finalize migration
      this._finishLoad(doc, file, "0.8");
      return;
    };
    
         
    /*::::: do complex migration :::::*/     
    //determine if we are in toolkit
    var isToolkit = true;
    try {
      var em = Components.classes["@mozilla.org/extensions/manager;1"].getService(Components.interfaces.nsIExtensionManager);
    } catch (e) {
       isToolkit = false;
    };

    //non toolkit version
    if (!isToolkit) {
    
      //get profile document
      doc = this._getDoc(file);
      if (!doc)
        return;
        
     // bug in version 0.7.0 and 0.7.1 for non toolkit did not update migrate pref
     // so manually override it here if its null
      if (exists && from == "")        
      doc = this._transform(doc, "0.7.1");
      if (!doc) {
        this._restart = this._createError(ffIError.SEVERITY_ERROR, "ff.transform", null, null);
        return;
      };
            
      //finalize migration
      this._finishLoad(doc, file, "0.8");
      return;
    };

    //toolkit version     
    //determine if we are coming from weatherfox
    var isWeatherfox = false;
    var weatherfox = new Array();
    try {
      weatherfox[0] = em.getItemForID("{BA8E053E-2867-4772-B9F0-26A5979AFA09}");
      isWeatherfox = (weatherfox[0].location != null);
    } catch(e) {
      weatherfox = em.getItemList("{BA8E053E-2867-4772-B9F0-26A5979AFA09}", Components.interfaces.nsIUpdateItem.TYPE_EXTENSION, {});
      isWeatherfox = (weatherfox[0] != null);
    };
        
    //not weatherfox
    if (!isWeatherfox) {
    
      //get profile document
      doc = this._getDoc(file);
      if (!doc)
        return;
            
      //has to be either 0.5.8 or 0.5.9 
      //so error on lower side or doesn't exist
      if (exists)
        doc = this._transform(doc, "0.5.8");                 
      if (!doc) {
        this._restart = this._createError(ffIError.SEVERITY_ERROR, "ff.transform", null, null);
        return;
      };
            
      //finalize migration
      this._finishLoad(doc, file, "0.8");
      return;
    };
                                          
    //is weatherfox
    //get profile document
    doc = this._getDoc(file);
    if (!doc)
      return;
            
    //can't transform a version before 0.4.8
    var vchecker = Components.classes["@mozilla.org/updates/version-checker;1"].getService(Components.interfaces.nsIVersionChecker);        
    if (vchecker.compare(weatherfox[0].version, "0.4.8") < 0)
      this._restart = this._createError(ffIError.SEVERITY_ERROR, "ff.migrate.version", null, null);
    else {
      //transform the weatherfox profile
      var wfile = this._manager.disk.get("profiles.xml", ffIDisk.TYPE_WEATHERFOX);
      if (wfile.exists() && wfile.isReadable()) {
        var wDoc = this._manager.disk.read(wfile);
        if (this._manager.disk.valid(wDoc)) {
          doc = this._transform(wDoc, weatherfox[0].version);
          if (!doc) {
            this._restart = this._createError(ffIError.SEVERITY_ERROR, "ff.transform", null, null);
            return;
          };
          //set current profile
          var pbs = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefService)
          var branch = pbs.getBranch("weatherfox.");
          this._branch.setCharPref("profile.current", branch.getCharPref("profile"));             
        };          
      };        
    };
         
    //remove weatherfox from em 
    try {
      // use this 1.1 and on
      em.uninstallItem("{BA8E053E-2867-4772-B9F0-26A5979AFA09}");
    } catch (e) {
      // use this through 1.0
      em.uninstallExtension("{BA8E053E-2867-4772-B9F0-26A5979AFA09}");
    };
            
    //finalize migration
    this._finishLoad(doc, file, "*");
    if (this._restart.severity != ffIError.SEVERITY_ERROR)                    
      this._restart = this._createError(ffIError.SEVERITY_WARNING, "ff.migrate.restart", null, null);
  },
  
  _finishLoad: function(aDoc, aFile, aVersion)
  {
    this._doc = aDoc;
    this._manager.disk.write(this._doc, aFile, true);
    this._branch.setCharPref("migrated", aVersion); 
    this._branch.lockPref("migrated");
    var profile;
    if (!this._isProfile(this._branch.getCharPref("profile.current"))) {
      profile = this._branch.getCharPref("profile.current");              
      this.create(profile);
    };
    profile = this._firstProfile();
    this.save(profile);    
    this.load(profile);
    this._batch = false;    
  },
  
  _getDoc: function(aFile)
  {
    var backup = this._manager.disk.get("profiles.bak", ffIDisk.TYPE_CACHE);
    var base = this._manager.disk.get("profiles.xml", ffIDisk.TYPE_DEFAULTS);
    var doc = null;
    
    //try loading profile file first
    if (aFile.exists()) {
      if (!aFile.isReadable() || !aFile.isWritable()) {
        this._restart = this._createError(ffIError.SEVERITY_ERROR, "ff.migrate.read", null, null);
        return null;
      };
      doc = this._manager.disk.read(aFile);
      if (this._manager.disk.valid(doc))
        return doc;
      else {
        // the profiles.xml file was corrupt - create an error for it
        this._manager.disk.recordErrorFile(aFile);
      }
    } else {
      this._manager.disk.recordErrorMsg("Expected profiles.xml, but was missing");
    };
    
    //try loading backup
    if (backup.exists() && backup.isReadable()) {    
      doc = this._manager.disk.read(backup);
      if (this._manager.disk.valid(doc)) {
        this._restart = this._createError(ffIError.SEVERITY_WARNING, "ff.migrate.backup", null, null);       
        return doc;    
      } else {
        // the backup file was corrupt - create an error for it
        this._manager.disk.recordErrorFile(backup);
      };
    } else {
      this._manager.disk.recordErrorMsg("Expected backup profiles.xml, but was missing");
    };
    
    //try loading default
    if (base.exists() && base.isReadable()) {
      doc = this._manager.disk.read(base);
      if (this._manager.disk.valid(doc)) {
        this._manager.disk.clear(ffIDisk.TYPE_CACHE, true);      
        this._restart = this._createError(ffIError.SEVERITY_INFO, "ff.migrate.default", null, null);
        return doc;    
      } else {
        // corrupt defaults file!
        this._manager.disk.recordErrorFile(base);
      };
    } else {
      // no defaults file!
      this._manager.disk.recordErrorMsg("WARNING: Expected default profiles.xml, but was missing!");
    };
 
    this._restart = this._createError(ffIError.SEVERITY_WARNING, "ff.migrate", null, null);      
    return null;    
  },
   
  _transform: function(aDoc, aFrom)
  {
    var processor = Components.classes["@mozilla.org/document-transformer;1?type=xslt"].createInstance(Components.interfaces.nsIXSLTProcessor);  
    var from = aFrom;
          
    //loop up through version
    while (from != "0.8") {

      //invalid document
      if (!this._manager.disk.valid(aDoc)) {
        return null;
      };
    
      //load the stylesheet
      var sheet = this._manager.disk.get("profiles-" + from + ".xsl", ffIDisk.TYPE_TRANSFORMS);
      if (!sheet.exists() || !sheet.isReadable()) {
        return null;
      };
      
      //invalid stylesheet         
      var sdoc = this._manager.disk.read(sheet);
      if (!this._manager.disk.valid(sdoc)) {
        return null;
      };
    
      //transform document
      processor.reset();
      processor.importStylesheet(sdoc);
      var frag = processor.transformToFragment(aDoc.getElementsByTagName("profiles")[0], aDoc);
      aDoc.replaceChild(frag.firstChild, aDoc.getElementsByTagName("profiles")[0]);
           
      //get version doc was transformed to
      var node = this._evalPath(aDoc, "//@version");
      if (!node) {
        return null;
      } else
        from = node.textContent;
    };
    
    return aDoc;
  },
  
  _import: function(aAlerts, aWindow, aFile)
  { 
    //load import file
    if (!aFile.exists()) {
      aAlerts.alert(aWindow, this._bundle.GetStringFromName("ff.import"), this._bundle.GetStringFromName("ff.import.document"));
      return;
    };   
    if (!aFile.isReadable) {
      aAlerts.alert(aWindow, this._bundle.GetStringFromName("ff.import"), this._bundle.GetStringFromName("ff.import.read"));
      return;
    };
    var doc = this._manager.disk.read(aFile);
    if (!this._manager.disk.valid(doc)) {
      aAlerts.alert(aWindow, this._bundle.GetStringFromName("ff.import"), this._bundle.GetStringFromName("ff.import.document"));
      return;    
    };
       
    //have current and profiles
    var current = doc.getElementsByTagName("current")[0];
    var profiles = doc.getElementsByTagName("profiles")[0];
    if (!current || !profiles) {
      aAlerts.alert(aWindow, this._bundle.GetStringFromName("ff.import"), this._bundle.GetStringFromName("ff.import.invalid"));
      return;    
    };
        
    //get version we are importing from
    var version = this._evalPath(doc, "//@version");
    if (!version) {
      aAlerts.alert(aWindow, this._bundle.GetStringFromName("ff.import"), this._bundle.GetStringFromName("ff.import.invalid"));
      return;        
    } else
      version = version.textContent;
     
    //get profiles node in current doc
    //and replace with profiles node in import doc
    var cDoc = this._doc.cloneNode(true);
    var node = this._evalPath(cDoc, "//profiles");
    var parent = node.parentNode;
    parent.removeChild(node);
    parent.appendChild(profiles.cloneNode(true));
        
    //tranform document if version is current
    if (version != "0.8") {
      cDoc = this._transform(cDoc, version);
      if (!cDoc) {
        aAlerts.alert(aWindow, this._bundle.GetStringFromName("ff.import"), this._bundle.GetStringFromName("ff.transform.tooltip"));      
        return;                  
      };
    };
        
    //now apply values from current node
    this._batch = true;
    prefs = current.getElementsByTagName("pref");
    for (x=0; x<prefs.length; x++) {
      if (!this._excludes[prefs[x].getAttribute("name")])
        continue;    

      if (this._branch.prefIsLocked(prefs[x].getAttribute("name")))
        continue;
                
      this._setPref(prefs[x]);
    };
    this._batch = false;
        
    //flush data to disk
    var file = this._manager.disk.get("profiles.xml", ffIDisk.TYPE_CACHE);
    this._manager.disk.write(cDoc, file, true); 
        
    //load profile
    this._current = null;
    this._doc = cDoc;    
    this.load(this._firstProfile());
    aAlerts.alert(aWindow, this._bundle.GetStringFromName("ff.import"), this._bundle.GetStringFromName("ff.import.success"));      
  },
  
  _export: function(aAlerts, aWindow, aFile)
  {
    //get export document
    var file = this._manager.disk.get("export.xml", ffIDisk.TYPE_DEFAULTS);
    if (!file.exists() || !file.isReadable()) {
      aAlerts.alert(aWindow, this._bundle.GetStringFromName("ff.export"), this._bundle.GetStringFromName("ff.export.document"));
      return;
    };
    var doc = this._manager.disk.read(file);
    if (!this._manager.disk.valid(doc)) {
      aAlerts.alert(aWindow, this._bundle.GetStringFromName("ff.export"), this._bundle.GetStringFromName("ff.export.document"));
      return;
    };
    
    //copy profiles to export document
    var profiles = this._evalPath(this._doc, "//profiles");
    var node = doc.getElementsByTagName("forecastfox-settings")[0];
    node.appendChild(profiles.cloneNode(true));
    
    //create a current node
    current = doc.createElement("current");
        
    //add preference to it
    prefs = this._getPrefs(); 
    for (var x=0; x<prefs.length; x++) {
      if (!this._excludes[prefs[x]])
        continue;    

      if (this._branch.prefIsLocked(prefs[x]))
        continue;
        
      try {
        var type = this._getPrefType(prefs[x]);
        var pref = this._doc.createElement("pref");
        pref.setAttribute("name", prefs[x]);
        pref.setAttribute("type", type);
        pref.setAttribute("value", eval("this._branch.get" + type + "Pref('" + prefs[x] + "')"));
        current.appendChild(pref);
      } catch(e) {};
    };
    
    //append current
    node.appendChild(current);
        
    //flush to disk
    if (!aFile.isWritable) {
      aAlerts.alert(aWindow, this._bundle.GetStringFromName("ff.export"), this._bundle.GetStringFromName("ff.export.write"));
      return;
    };
    
    //finished    
    this._manager.disk.write(doc, aFile, false);
    aAlerts.alert(aWindow, this._bundle.GetStringFromName("ff.export"), this._bundle.GetStringFromName("ff.export.success"));
  },
  
  _firstProfile: function()
  {    
    //current profile set and valid
    var profile = this._current;
    if (profile && this._isProfile(profile))
      return profile;
    
    //preference set and valid
    profile = this._branch.getCharPref("profile.current");
    if (profile && this._isProfile(profile))
      return profile;
      
    //default profile exists
    profile = "default";
    if (profile && this._isProfile(profile))
      return profile;
    
    //grab first profile available
    profile = this._doc.getElementsByTagName("profile")[0].getAttribute("name");
    if (profile && this._isProfile(profile))
      return profile;
      
    return null;
  },
      
  _createError: function(aSeverity, aName, aDescription, aTooltip)
  {
    var description = (aName) ? this._bundle.GetStringFromName(aName + ".label") : aDescription;
    var tooltip = (aName) ? this._bundle.GetStringFromName(aName + ".tooltip") : aTooltip;
    var error = Components.classes["@ensolis.com/forecastfox/error;1"].createInstance(Components.interfaces.ffIError);
    error.init(aSeverity, "@ensolis.com/forecastfox/profiles;1", description, tooltip); 
    return error;
  },

  _filePicker: function(aOpen, aWindow)
  {
    var picker = Components.classes["@mozilla.org/filepicker;1"].createInstance(nsIFilePicker);  
    picker.appendFilters(nsIFilePicker.filterXML);
    picker.defaultExtension = ".xml";
    switch (aOpen) {
      case ffIProfiles.OPEN_EXPORT:
        picker.init(aWindow, this._bundle.GetStringFromName("ff.export"), nsIFilePicker.modeSave);
        break;
      case ffIProfiles.OPEN_IMPORT:
        picker.init(aWindow, this._bundle.GetStringFromName("ff.import"), nsIFilePicker.modeOpen);
        break;
    };
    
    // get the file and its contents
    var res = picker.show();
    if (res == nsIFilePicker.returnCanel)
      return null;
    else
      return picker.file;
  },
  
  _isProfile: function(aName)
  {
    var profile = this._evalPath(this._doc, "//profiles/profile[@id='"+aName+"']");
    return (profile != null);
  },
  
  _schedule: function()
  {
    if (!this._rotating)
      this.rotate(ffIProfiles.ROTATE_STOP);
    else {
      //see if we have multiple profiles
      var profiles = this._evalPath(this._doc, "//profiles");
      if (profiles.getElementsByTagName("profile").length <= 1)
        return;

      var rotate = this._branch.getBoolPref("profile.switch.enabled");
      var rotateTime = this._branch.getIntPref("profile.switch.delay");
    
      //bail out if its disabled
      if (rotate == false || rotateTime == 0)
        return;

      rotateTime *= 1000 * 60;

      // then schedule the timer
      this._timer.initWithCallback(this, rotateTime, Components.interfaces.nsITimer.TYPE_ONE_SHOT);
    }     
  },
  
  _encodeValue: function(aValue)
  {
    // only encode the strings
    if (typeof aValue != "string")
      return aValue;
    
    // prepare the quotes so that we can use eval
    return aValue.replace(/"/g,"\\\"").replace(/'/g,"\\\'");
  },
    
  _setPref: function(aNode)
  {
    if (!aNode)
      return;
      
    //get variables
    var pref  = aNode.getAttribute("name");
    var type  = aNode.getAttribute("type");
    var value = aNode.getAttribute("value");
    
    //set value based on type
    if (type == "Bool")
      value = (value == "true") ? true : false;
    else if (type == "Int")
      value = Number(value);
    else if (type == "Char")
      value = value

    //get the old value
    var oldValue = null;
    var defaultValue = null;
    var hasValue = false;
    try {
      oldValue = eval("this._branch.get" + type + "Pref('" + pref + "')");
      defaultValue = eval("this._default.get" + type + "Pref('" + pref + "')");
      hasValue = eval("this._branch.prefHasUserValue('" + pref + "')");
    } catch(e) {};
    
    //do nothing if the same value
    if (value == oldValue)
      return;
    
    //clear if same as default value
    if (value == defaultValue && hasValue)
      eval("this._branch.clearUserPref('" + pref + "')");
    else {
      if (type == "Char")
        value = '"' + this._encodeValue(value) + '"'; 
      eval("this._branch.set" + type + "Pref('" + pref + "', " + value + ")");
    }
  },
    
  _getPrefType: function(aName)
  {
    //convert integer type to character type
    var type = this._branch.getPrefType(aName);
    var val = null;
    switch (type) {
      case 0:
        val = "";
        break;
      case 32:
      default:
        val = "Char";
        break;
      case 64:
        val = "Int";
        break;
      case 128:
        val = "Bool";
        break;
    };
    
    return val;
  },
    
  _evalPath: function(aNode, aExpr)
  {
    var result = null;
     
    try {   
      var results = this._xpath.evaluate(aExpr, aNode, this._xresolve, nsIDOMXPathResult.ANY_TYPE, null);
      result = results.iterateNext(); 
    } catch(e) {};
    
    return result;    
  },
      
  _cacheFiles: function(aFrom, aTo, aDelete)
  {
    var junk = this._manager.disk.get("xyz", ffIDisk.TYPE_CACHE);
    var folder = junk.parent;
    var i, to, name;
    var fName = "";
    var tName = "";
    
    //get names  
    fName = aFrom.getAttribute("id");
    if (aTo)
      tName = aTo.getAttribute("id");
      
    //get a list of cache files
    var entries = folder.directoryEntries;
    var files = new Array();
    while (entries.hasMoreElements()) {
      var entry = entries.getNext();
      entry = entry.QueryInterface(Components.interfaces.nsIFile);
      name = entry.leafName;
      
      //make sure its a cache file
      if (name.indexOf("cache") == -1)
        continue;
        
      //make sure it is a from cache
      if (name.indexOf(fName) != -1)
        files.push(entry);
    };

    //now loop through our files    
    for (i=0; i<files.length; i++) {
      
      //get to file
      if (aTo) {
        to = folder.clone();
        name = files[i].leafName;
        name = name.replace(fName, tName);       
        to.append(name);
        if (files[i].exists())
          this._manager.disk.copy(files[i], to);
      };
      
      //delete file
      if (aDelete && files[i].exists())
        files[i].remove(false);
    }  
  },
  
  _cachePrefs: function(aFrom, aTo)
  {
    //get a list of preferences
    var prefs = this._getPrefs();
    var nodes = new Object();
    
    //loop through prefs finding cache prefs
    for (var i=0; i<prefs.length; i++) {
      if (prefs[i].indexOf("cache") == -1)
        continue;
      
      //get profile node that matches
      nodes[prefs[i]] = this._evalPath(aTo, "./pref[@name='" + prefs[i] + "']");
    };

    //replace value and update
    //pref may not exist if we migrated so wrap in try catch
    var fName = aFrom.getAttribute("id");
    var tName = aTo.getAttribute("id");
    for (j in nodes) {
      try {
        var value = nodes[j].getAttribute("value");
        value = value.replace(fName, tName);
        nodes[j].setAttribute("value", value);
      } catch(e) {};
    }
  },
  
  _getPrefs: function(aExcludes)
  {
    var file = this._manager.disk.get("profiles.xml", ffIDisk.TYPE_DEFAULTS);
    var doc = this._manager.disk.read(file);
    if (!this._manager.disk.valid(doc)) {
      // it is invalid!
      this._manager.disk.recordErrorMsg("Error getting defaults profiles.xml for _getPrefs.");
      return null;
    }
    
    var prefNodes = doc.getElementsByTagName("pref");
    var prefs = new Array();
    
    for (var i = 0; i < prefNodes.length; i++)
      prefs[i] = prefNodes[i].getAttribute("name");
      
    if (aExcludes) {
      for (pref in aExcludes)
        prefs[i++] = pref;
    }

    return prefs;
  },
        
  _removeBranch: function(aBranch, aReset)
  {
    var pbs = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefService)
    var branch = pbs.getBranch(aBranch);

    //do a reset
    if (aReset) {   
      //get list of prefs
      var prefs = branch.getChildList("", {});
      
      for (var x=0; x<prefs.length; x++) {    
        //clear user value
        if (branch.prefHasUserValue(prefs[x])) {        
          if (this._excludes[prefs[x]])
            continue;
          else
            branch.clearUserPref(prefs[x]);  
        };
      };
    } else    
      branch.deleteBranch("");
  },

  _stripName: function(name)
  {
    // remove illegal characters
    return name.replace(/[\\\/:*?<>|\'\"]/g, "")
  },
  
  ///////////////////////////
  // nsITimerCallback
  notify: function(timer)
  {
    var profiles = this._evalPath(this._doc, "//profiles");  
    profiles = profiles.getElementsByTagName("profile");
    
    // go to next profile in the list
    for (var x=0; x<profiles.length; x++) {
      if (profiles[x].getAttribute("id") == this._current) {
        this._branch.setCharPref("profile.current", profiles[( x+1 < profiles.length ) ? x+1 : 0 ].getAttribute("id"));
        break;
      };
    };
    
    //schedule rotation
    this._schedule();
  },
       
  ///////////////////////////
  // nsISupports
  QueryInterface: function (aIID)
  {
    if (!aIID.equals(Components.interfaces.ffIProfiles) &&
        !aIID.equals(Components.interfaces.nsISupports))
      throw Components.results.NS_ERROR_NO_INTERFACE;
    return this;
  }
};

/******************************************************************************
 * XPCOM Functions for construction and registration
 ******************************************************************************/
var gModule = {
  _firstTime: true,
  registerSelf: function(aCompMgr, aFileSpec, aLocation, aType)
  {
    if (this._firstTime) {
      this._firstTime = false;
      throw Components.results.NS_ERROR_FACTORY_REGISTER_AGAIN;
    };
    aCompMgr = aCompMgr.QueryInterface(Components.interfaces.nsIComponentRegistrar);
    aCompMgr.registerFactoryLocation(CLASS_ID, CLASS_NAME, CONTRACT_ID, aFileSpec, aLocation, aType);
  },

  unregisterSelf: function(aCompMgr, aLocation, aType)
  {
    aCompMgr = aCompMgr.QueryInterface(Components.interfaces.nsIComponentRegistrar);
    aCompMgr.unregisterFactoryLocation(CLASS_ID, aLocation);        
  },
  
  getClassObject: function(aCompMgr, aCID, aIID)
  {
    if (!aIID.equals(Components.interfaces.nsIFactory))
      throw Components.results.NS_ERROR_NOT_IMPLEMENTED;

    if (aCID.equals(CLASS_ID))
      return gFactory;

    throw Components.results.NS_ERROR_NO_INTERFACE;
  },

  canUnload: function(aCompMgr) { return true; }
};

var gFactory = {
  createInstance: function (aOuter, aIID)
  {
    if (aOuter != null)
      throw Components.results.NS_ERROR_NO_AGGREGATION;
    return (new ffProfiles()).QueryInterface(aIID);
  }
};

function NSGetModule(aCompMgr, aFileSpec) { return gModule; }