/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Forecastfox.
 *
 * The Initial Developer of the Original Code is
 * Jon Stritar <jstritar@MIT.EDU>.
 * Portions created by the Initial Developer are Copyright (C) 2005
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 * Jon Stritar <jstritar@MIT.EDU>
 * Richard Klein <richwklein@mchsi.com>
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

var gProfiles = null;

function profilesLoad()
{
  gProfiles = new ProfilesModule();
  gProfiles.start();
};

function profilesUnload()
{
  gProfiles.stop();
  gProfiles = null;
};

function ProfilesModule()
{
  this.wrappedJSObject = this;
  return this;
};

ProfilesModule.prototype = {
  _manager: null,
  _type: null,
  _bundle: null,
  _textbox: null,
  _branch: null,
  _prompt: null,
  _guid: null,
  
  start: function()
  {
    var pbs = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefService);   
    this._manager = Components.classes["@ensolis.com/forecastfox/manager;1"].getService(Components.interfaces.ffIManager);
    this._type = window.arguments[0];
    this._bundle = document.getElementById("ff-bundle-profiles");  
    this._textbox = document.getElementById("ff-text-name");
    this._branch = pbs.getBranch("forecastfox.");
    
    try {
      var app = Components.classes["@mozilla.org/xre/app-info;1"].getService(Components.interfaces.nsIXULAppInfo);    
      this._guid = app.ID;
    } catch(e) {
      var appBranch = pbs.getBranch(null);
      try {
        this._guid = appBranch.getCharPref("app.id");
      } catch(e) {
        this._guid = "{86c18b42-e466-45a9-ae7a-9b95ba6f5640}";
      };    
    };  
          
    //set description
    document.getElementById("ff-desc-type").value = this._bundle.getString("ff.profiles." + this._type);
    switch (this._type) {
      case "create":
        this._textbox.value = "";
        break;
      case "rename":
        this._textbox.value = this._branch.getCharPref("profile.current");
        break;
      case "remove":
        this._textbox.disabled = true;
        this._textbox.value = this._branch.getCharPref("profile.current");
        break;   
    };
    
    //set focus
    if (this._type != "remove") {
      this._textbox.focus();
      this._textbox.select();
    }  
  },
  
  stop: function()
  {
    this._manager = null;  
    this._type = null;
    this._bundle = null;
    this._textbox = null;
    this._branch = null;
    this._prompt = null; 
    this._guid = null;
  },

  accept: function()
  {
    try {
      var x;
      var names = this._manager.profiles.getProfiles({});
      
      if (!this._validate())
        return;
        
      switch (this._type) {
        case "create":
          this._manager.profiles.create(this._textbox.value);
          break;
        case "rename":
          this._manager.profiles.rename(this._branch.getCharPref("profile.current"), this._textbox.value);
          break;         
        case "remove":
          //switch to different profile
          for (x=0; x<names.length; x++) {
            if (names[x] != this._textbox.value)
              break;   
          };
          this._manager.profiles.remove(this._textbox.value, names[x]);        
          break;         
      };
     
      window.close();
    } catch (e) {
      this._manager.disk.recordErrorMsg("An error has occured while "+ this._type.substring(0,this._type.length -1)+
                " a profile. Exception contents: "+e);
      var strings = document.getElementById("ff-bundle-profiles");
      alert(strings.getString("ff.error.profile")+"\n"+strings.getString("ff.error.contact"));
    }
  },
  
  _validate: function()
  {
    var x;
    var names = this._manager.profiles.getProfiles({});
    
    switch (this._type) {
      case "create":
        //check if valide profile name entered
        if (this._textbox.value == "") {
          this._alert("ff.profiles.invalid");
          return false;
        };
    
        //check if a profile by this name already exists
        for (x=0; x < names.length; x++ ) {
          if (names[x] == this._textbox.value) {
            this._alert("ff.profiles.exists");
            return false;
          };  
        };
        
        break;
      case "rename":
        //check if valide profile name entered
        if (this._textbox.value == "") {
          this._alert("ff.profiles.invalid");
          return false;
        };
    
        //check if a profile by this name already exists
        for (x=0; x < names.length; x++ ) {
          if (names[x] == this._textbox.value) {
            this._alert("ff.profiles.exists");
            return false;
          };  
        };      
        break;         
      case "remove":
        //check that we aren't the last profile
        if (names.length == 1) {
          this._alert("ff.profiles.last");
          return false; 
        };
        break;      
    };
    
    return true;
  },
  
  _alert: function(aText)
  {
    if (!this._prompt)
      this._prompt = Components.classes["@mozilla.org/embedcomp/prompt-service;1"].getService(Components.interfaces.nsIPromptService);
    this._prompt.alert(window, this._bundle.getString("ff.profiles"), this._bundle.getString(aText));
  },
  
  openLink: function(aEvent, aPartner)
  {
    var url = aEvent.currentTarget.getAttribute("url");
    if (aPartner)
      url = url + "?partner=" + this._manager.partner;
  
    //XXX need app specific way that doesn't use app.id pref
    if (this._guid != "{86c18b42-e466-45a9-ae7a-9b95ba6f5640}")
      window.openDialog("chrome://browser/content/browser.xul", "_blank", "chrome,all,dialog=no", url, null, null);
    else
      window.openDialog("chrome://navigator/content/navigator.xul", "_blank", "chrome,all,dialog=no", url, null, null);
  }  
}